package cn.com.ctop.common.module.achievements.controller;

import cn.com.ctop.common.module.achievements.service.IDesignerService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 设计人员绩效
 *
 * @author: zian Y
 * @date: 2022-09-28
 * @cersion: V1.0
 */
@Slf4j
@Api(tags = "设计人员绩效")
@RestController
@RequestMapping("/designer/designerAchievementsController")
public class DesignerAchievementsController {
    @Autowired
    private IDesignerService designerService;

    /**
     * 查询设计人员基础信息
     *
     * @param userId
     * @return
     */
    @GetMapping(value = "/getDesignBasicsInfoList")
    public PageInfo<JSONObject> getDesignBasicsInfoList(@RequestParam(name = "userId") String userId,
                                                        @RequestParam(name = "pageNum",defaultValue = "1") Integer pageNum,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {

        return designerService.getDesignBasicsInfoList(userId,pageNum,pageSize);

    }

    /**
     * 爆款视频数据
     *
     * @param userId
     * @param
     * @return
     */
    @GetMapping(value = "/getHotVideoData")
    public List<Map<String,Object>> getHotVideoData(@RequestParam(name = "userId") String userId,
                                                    @RequestParam(name = "yearTime")String yearTime,
                                                    @RequestParam(name = "monthTime")String monthTime) {

        return designerService.getHotVideoData(userId,yearTime,monthTime);

    }

    /**
     * 设计绩效
     *
     * @param userId
     * @param
     * @return
     */
    @GetMapping(value = "/getDesignerAchievements")
    public Map<String,Object> getDesignerAchievements(@RequestParam(name = "userId") String userId,
                                                      @RequestParam(name = "yearTime")String yearTime,
                                                      @RequestParam(name = "monthTime")String monthTime) {
        return designerService.getDesignerAchievements(userId,yearTime,monthTime);

    }

    /**
     * 设计提成
     *
     * @param userId
     * @param
     * @return
     */
    @GetMapping(value = "/getDesignerCommission")
    public JSONObject getDesignerCommission(@RequestParam(name = "userId") String userId,
                                            @RequestParam(name = "yearTime")String yearTime,
                                            @RequestParam(name = "monthTime")String monthTime) {
        return designerService.getDesignerCommission(userId,yearTime,monthTime);

    }


    /**
     * 修改绩效中的用户部分信息
     *
     * @param jsonObject
     * @param
     * @return
     */
    @PostMapping(value = "/updateUserHappyProbability")
    public Result updateUserHappyProbability(@RequestBody JSONObject jsonObject) {

        return designerService.updateUserHappyProbability(jsonObject);

    }


    /**
     * 基础信息+爆款
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/getDesignerBasicsAndHotVideoInfo")
    public Result getDesignerCommission(@RequestParam(name = "userId") String userId,
                                        @RequestParam(name = "yearTime") String yearTime,
                                        @RequestParam(name = "monthTime") String monthTime,
                                        @RequestParam(name = "pageNum",defaultValue = "1") Integer pageNum,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
       return designerService.getDesignerBasicsAndHotVideoInfo(userId,yearTime,monthTime,pageNum,pageSize);

    }



    /**
     * 基础信息+绩效
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/getDesignerBasicsAndAch")
    public Result getDesignerBasicsAndAch(@RequestParam(name = "userId") String userId,
                                                    @RequestParam(name = "yearTime") String yearTime,
                                                    @RequestParam(name = "monthTime") String monthTime,
                                        @RequestParam(name = "pageNum",defaultValue = "1") Integer pageNum,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
       return designerService.getDesignerBasicsAndAch(userId,yearTime,monthTime,pageNum,pageSize);

    }


    /**
     * 基础信息+提成
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @GetMapping(value = "/getDesignerBasicsAndAchAndComInfo")
    public Result getDesignerBasicsAndAchAndComInfo(@RequestParam(name = "userId") String userId,
                                                    @RequestParam(name = "yearTime") String yearTime,
                                                    @RequestParam(name = "monthTime") String monthTime,
                                        @RequestParam(name = "pageNum",defaultValue = "1") Integer pageNum,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
       return designerService.getDesignerBasicsAndAchAndComInfo(userId,yearTime,monthTime,pageNum,pageSize);

    }


    /**
     * 根据角色查询人员
     * @param roleName
     * @return
     */
    @GetMapping(value = "/getUserByRoleName")
    public Result getUserByRoleName(@RequestParam(name = "roleName") String roleName) {
       return designerService.getUserByRoleName(roleName);

    }



}
