package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAccountAutoVideo;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAccountAutoVideoService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 自动投放账户——素材
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-10-20
 */
@Slf4j
@Api(tags = "自动投放账户——素材")
@RestController
@RequestMapping("/auto/aiKuaishouAccountAutoVideo")
public class AiKuaishouAccountAutoVideoController {
    @Autowired
    private IAiKuaishouAccountAutoVideoService aiKuaishouAccountAutoVideoService;

    /**
     * 分页列表查询
     *
     * @param aiKuaishouAccountAutoVideo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-分页列表查询", notes = "自动投放账户——素材-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AiKuaishouAccountAutoVideo>> queryPageList(AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideo,
                                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<AiKuaishouAccountAutoVideo>> result = new Result<>();
        QueryWrapper<AiKuaishouAccountAutoVideo> queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouAccountAutoVideo, req.getParameterMap());
        Page<AiKuaishouAccountAutoVideo> page = new Page<AiKuaishouAccountAutoVideo>(pageNo, pageSize);
        IPage<AiKuaishouAccountAutoVideo> pageList = aiKuaishouAccountAutoVideoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 自动投放重构清洗账户素材
     */
    @GetMapping(value = "/insertAutoVideo")
    public Result<AiKuaishouAccountAutoVideo> insertAutoVideo(Long accountId) {
        Result<AiKuaishouAccountAutoVideo> result = new Result<>();
        try {
            aiKuaishouAccountAutoVideoService.insertAutoVideo(accountId);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 添加
     *
     * @param aiKuaishouAccountAutoVideo
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-添加", notes = "自动投放账户——素材-添加")
    @PostMapping(value = "/add")
    public Result<AiKuaishouAccountAutoVideo> add(@RequestBody AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideo) {
        Result<AiKuaishouAccountAutoVideo> result = new Result<>();
        try {
            aiKuaishouAccountAutoVideoService.save(aiKuaishouAccountAutoVideo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param aiKuaishouAccountAutoVideo
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-编辑", notes = "自动投放账户——素材-编辑")
    @PutMapping(value = "/edit")
    public Result<AiKuaishouAccountAutoVideo> edit(@RequestBody AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideo) {
        Result<AiKuaishouAccountAutoVideo> result = new Result<AiKuaishouAccountAutoVideo>();
        AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideoEntity = aiKuaishouAccountAutoVideoService.getById(aiKuaishouAccountAutoVideo.getId());
        if (aiKuaishouAccountAutoVideoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = aiKuaishouAccountAutoVideoService.updateById(aiKuaishouAccountAutoVideo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-通过id删除", notes = "自动投放账户——素材-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            aiKuaishouAccountAutoVideoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-批量删除", notes = "自动投放账户——素材-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AiKuaishouAccountAutoVideo> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<AiKuaishouAccountAutoVideo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.aiKuaishouAccountAutoVideoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放账户——素材-通过id查询", notes = "自动投放账户——素材-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AiKuaishouAccountAutoVideo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AiKuaishouAccountAutoVideo> result = new Result<>();
        AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideo = aiKuaishouAccountAutoVideoService.getById(id);
        if (aiKuaishouAccountAutoVideo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(aiKuaishouAccountAutoVideo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AiKuaishouAccountAutoVideo> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AiKuaishouAccountAutoVideo aiKuaishouAccountAutoVideo = JSON.parseObject(deString, AiKuaishouAccountAutoVideo.class);
                queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouAccountAutoVideo, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AiKuaishouAccountAutoVideo> pageList = aiKuaishouAccountAutoVideoService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "自动投放账户——素材列表");
        mv.addObject(NormalExcelConstants.CLASS, AiKuaishouAccountAutoVideo.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自动投放账户——素材列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AiKuaishouAccountAutoVideo> listAiKuaishouAccountAutoVideos = ExcelImportUtil.importExcel(file.getInputStream(), AiKuaishouAccountAutoVideo.class, params);
                aiKuaishouAccountAutoVideoService.saveBatch(listAiKuaishouAccountAutoVideos);
                return Result.ok("文件导入成功！数据行数:" + listAiKuaishouAccountAutoVideos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
