package cn.com.ctop.kuaishou.modules.clean.controller;

import cn.com.ctop.common.module.entity.Project;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IProjectService;
import cn.com.ctop.common.module.service.ISysUserService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.clean.entity.TrackMonitorConfigure;
import cn.com.ctop.kuaishou.modules.clean.service.ITrackMonitorConfigureService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.entity.SysUser;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.service.ISysRoleService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 监测链接预警-配置
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-09-23
 */
@Slf4j
@Api(tags = "监测链接预警-配置")
@RestController
@RequestMapping("/monitor/trackMonitorConfigure")
public class TrackMonitorConfigureController {
    @Autowired
    private ITrackMonitorConfigureService trackMonitorConfigureService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private IUserAllocationService allocationService;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private ISysUserService userService;


    /**
     * 分页列表查询
     *
     * @param trackMonitorConfigure
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-分页列表查询", notes = "监测链接预警-配置-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<TrackMonitorConfigure>> queryPageList(TrackMonitorConfigure trackMonitorConfigure,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<TrackMonitorConfigure>> result = new Result<>();

        String userId = trackMonitorConfigure.getUserId();
        String roleCode = sysRoleService.getRoleCodeByUserId(userId);
        if (roleCode.equals("admin")) {
            trackMonitorConfigure.setUserId(null);
        }
        QueryWrapper<TrackMonitorConfigure> queryWrapper = QueryGenerator.initQueryWrapper(trackMonitorConfigure, req.getParameterMap());
        Page<TrackMonitorConfigure> page = new Page<TrackMonitorConfigure>(pageNo, pageSize);
        IPage<TrackMonitorConfigure> pageList = trackMonitorConfigureService.page(page, queryWrapper);
        List<TrackMonitorConfigure> records = pageList.getRecords();
        for (TrackMonitorConfigure configure : records) {
            Long accountId = configure.getAccountId();
            if (!Check.isNull(accountId)) {
                UserAllocation userAllocation = allocationService.getByAccountId(accountId);
                if (!Check.isNull(userAllocation)) {
                    configure.setAccountName(userAllocation.getAuthName());
                    configure.setAccountStatus(userAllocation.getAccountStatus());
                }
            }
            Long projectId = configure.getProjectId();
            if (!Check.isNull(projectId)) {
                Project project = projectService.getById(projectId);
                if (!Check.isNull(project)) {
                    configure.setProjectName(project.getProjectName());
                }
            }
            String createUserId = configure.getUserId();
            if (!Check.isNull(createUserId)) {
                SysUser user = userService.getById(createUserId);
                if (!Check.isNull(user)) {
                    configure.setUserName(user.getRealname());
                }
            }
        }

        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param trackMonitorConfigure
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-添加", notes = "监测链接预警-配置-添加")
    @PostMapping(value = "/add")
    public Result<TrackMonitorConfigure> add(@RequestBody TrackMonitorConfigure trackMonitorConfigure) {
        Result<TrackMonitorConfigure> result = new Result<>();
        try {
            Long accountId = trackMonitorConfigure.getAccountId();
            QueryWrapper<TrackMonitorConfigure> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("account_id", accountId);
            queryWrapper.last("limit 1");
            TrackMonitorConfigure one = trackMonitorConfigureService.getOne(queryWrapper);
            if (!Check.isNull(one)) {
                result.error500("该账户预警已绑定");
                return result;
            }
            UserAllocation userAllocation = allocationService.getByAccountId(accountId);
            if (Check.isNull(userAllocation)) {
                result.error500("该账户未获取到关联项目");
                return result;
            }
            trackMonitorConfigure.setProjectId(userAllocation.getProjectId());
            trackMonitorConfigureService.save(trackMonitorConfigure);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param trackMonitorConfigure
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-编辑", notes = "监测链接预警-配置-编辑")
    @PostMapping(value = "/edit")
    public Result<TrackMonitorConfigure> edit(@RequestBody TrackMonitorConfigure trackMonitorConfigure) {
        Result<TrackMonitorConfigure> result = new Result<TrackMonitorConfigure>();
        TrackMonitorConfigure trackMonitorConfigureEntity = trackMonitorConfigureService.getById(trackMonitorConfigure.getId());
        if (trackMonitorConfigureEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = trackMonitorConfigureService.updateById(trackMonitorConfigure);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-通过id删除", notes = "监测链接预警-配置-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            trackMonitorConfigureService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-批量删除", notes = "监测链接预警-配置-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<TrackMonitorConfigure> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<TrackMonitorConfigure> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.trackMonitorConfigureService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-配置-通过id查询", notes = "监测链接预警-配置-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<TrackMonitorConfigure> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<TrackMonitorConfigure> result = new Result<>();
        TrackMonitorConfigure trackMonitorConfigure = trackMonitorConfigureService.getById(id);
        if (trackMonitorConfigure == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(trackMonitorConfigure);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<TrackMonitorConfigure> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                TrackMonitorConfigure trackMonitorConfigure = JSON.parseObject(deString, TrackMonitorConfigure.class);
                queryWrapper = QueryGenerator.initQueryWrapper(trackMonitorConfigure, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<TrackMonitorConfigure> pageList = trackMonitorConfigureService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "监测链接预警-配置列表");
        mv.addObject(NormalExcelConstants.CLASS, TrackMonitorConfigure.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("监测链接预警-配置列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<TrackMonitorConfigure> listTrackMonitorConfigures = ExcelImportUtil.importExcel(file.getInputStream(), TrackMonitorConfigure.class, params);
                trackMonitorConfigureService.saveBatch(listTrackMonitorConfigures);
                return Result.ok("文件导入成功！数据行数:" + listTrackMonitorConfigures.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
