package cn.com.ctop.kuaishou.modules.hosting.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.hosting.entity.KuaishouHostingTask;
import cn.com.ctop.kuaishou.modules.hosting.service.IKuaishouHostingTaskService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 智能托管-任务详情
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-11-09
 */
@Slf4j
@Api(tags = "智能托管-任务详情")
@RestController
@RequestMapping("/task/kuaishouHostingTask")
public class KuaishouHostingTaskController {
    @Autowired
    private IKuaishouHostingTaskService kuaishouHostingTaskService;
    private static ExecutorService createExecutorService = Executors.newFixedThreadPool(5);

    @PostMapping(value = "/submitTask")
    public Result<Object> submitTask(@RequestBody JSONObject requestJson) {
        Result<Object> result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("入参不能为空");
            }
            Long templateId = requestJson.getLong("templateId");
            if (Check.isNull(templateId)) {
                throw new Exception("请传入模板id");
            }
            JSONArray videoArray = requestJson.getJSONArray("videoArray");
            if (Check.isNull(videoArray)) {
                throw new Exception("请传入需创建的素材");
            }

            kuaishouHostingTaskService.submitTask(templateId, videoArray);

            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500(e.getMessage());
        }
        return result;
    }


    @GetMapping(value = "/createHosting")
    public JSONObject createHosting(Long id) {
        JSONObject returnJson = new JSONObject();
        try {
            if (Check.isNull(id)) {
                throw new Exception("入参不能为空");
            }
            kuaishouHostingTaskService.updateStatusById(id, 2, "智能托管计划创建中");
            createExecutorService.submit(new Runnable() {
                @Override
                public void run() {
                    kuaishouHostingTaskService.createHosting(id);
                }
            });
            returnJson.put("code", 0);
            returnJson.put("message", "提交成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            returnJson.put("code", -1);
            returnJson.put("message", "提交失败");
        }
        return returnJson;
    }


    /**
     * 分页列表查询
     *
     * @param kuaishouHostingTask
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-分页列表查询", notes = "智能托管-任务详情-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouHostingTask>> queryPageList(KuaishouHostingTask kuaishouHostingTask,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<KuaishouHostingTask>> result = new Result<>();
        QueryWrapper<KuaishouHostingTask> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouHostingTask, req.getParameterMap());
        queryWrapper.orderByDesc("create_time");
        Page<KuaishouHostingTask> page = new Page<KuaishouHostingTask>(pageNo, pageSize);
        IPage<KuaishouHostingTask> pageList = kuaishouHostingTaskService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaishouHostingTask
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-添加", notes = "智能托管-任务详情-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouHostingTask> add(@RequestBody KuaishouHostingTask kuaishouHostingTask) {
        Result<KuaishouHostingTask> result = new Result<>();
        try {
            kuaishouHostingTaskService.save(kuaishouHostingTask);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouHostingTask
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-编辑", notes = "智能托管-任务详情-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouHostingTask> edit(@RequestBody KuaishouHostingTask kuaishouHostingTask) {
        Result<KuaishouHostingTask> result = new Result<KuaishouHostingTask>();
        KuaishouHostingTask kuaishouHostingTaskEntity = kuaishouHostingTaskService.getById(kuaishouHostingTask.getId());
        if (kuaishouHostingTaskEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouHostingTaskService.updateById(kuaishouHostingTask);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-通过id删除", notes = "智能托管-任务详情-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            kuaishouHostingTaskService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-批量删除", notes = "智能托管-任务详情-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouHostingTask> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaishouHostingTask> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouHostingTaskService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "智能托管-任务详情-通过id查询", notes = "智能托管-任务详情-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouHostingTask> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouHostingTask> result = new Result<>();
        KuaishouHostingTask kuaishouHostingTask = kuaishouHostingTaskService.getById(id);
        if (kuaishouHostingTask == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouHostingTask);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouHostingTask> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouHostingTask kuaishouHostingTask = JSON.parseObject(deString, KuaishouHostingTask.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouHostingTask, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouHostingTask> pageList = kuaishouHostingTaskService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "智能托管-任务详情列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouHostingTask.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("智能托管-任务详情列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouHostingTask> listKuaishouHostingTasks = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouHostingTask.class, params);
                kuaishouHostingTaskService.saveBatch(listKuaishouHostingTasks);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouHostingTasks.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
