package cn.com.ctop.kuaishou.modules.hosting.controller;

import cn.com.ctop.common.module.entity.Project;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IProjectService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouVideoGet;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouVideoGetService;
import cn.com.ctop.kuaishou.modules.hosting.entity.KuaishouHostingTemplate;
import cn.com.ctop.kuaishou.modules.hosting.service.IKuaishouHostingTemplateService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.service.ISysRoleService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 智能托管模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-11-08
 */
@Slf4j
@Api(tags = "智能托管模板")
@RestController
@RequestMapping("/hosting/kuaishouHostingTemplate")
public class KuaishouHostingTemplateController {
    @Autowired
    private IKuaishouHostingTemplateService kuaishouHostingTemplateService;
    @Autowired
    private IKuaiShouVideoGetService videoGetService;
    @Autowired
    private ISysRoleService sysRoleService;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private IProjectService projectService;


    /**
     * 获取视频列表
     *
     * @return
     */
    @GetMapping(value = "/getVideoList")
    public Result<IPage<KuaiShouVideoGet>> getVideoList(@RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                        Long accountId,
                                                        String startDate,
                                                        String endDate, String name) {
        Result<IPage<KuaiShouVideoGet>> result = new Result<>();
        try {

            QueryWrapper<KuaiShouVideoGet> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("account_id", accountId);
            queryWrapper.eq("status", 0);
            if (!Check.isNull(startDate) && !Check.isNull(endDate)) {
                startDate += " 00:00:01";
                endDate += " 23:59:59";
                queryWrapper.ge("stat_date", startDate);
                queryWrapper.le("stat_date", endDate);
            }
            if (!Check.isNull(name)) {
                queryWrapper.like("photo_name", name);
            }
            queryWrapper.orderByDesc("stat_date");
            Page<KuaiShouVideoGet> page = new Page<>(pageNo, pageSize);
            IPage<KuaiShouVideoGet> pageList = videoGetService.page(page, queryWrapper);
            result.setSuccess(true);
            result.setResult(pageList);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
        }

        return result;
    }

    /**
     * 分页列表查询
     *
     * @param kuaishouHostingTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "智能托管模板-分页列表查询", notes = "智能托管模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouHostingTemplate>> queryPageList(KuaishouHostingTemplate kuaishouHostingTemplate,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<KuaishouHostingTemplate>> result = new Result<>();
        String userId = kuaishouHostingTemplate.getUserId();
        String roleCode = sysRoleService.getRoleCodeByUserId(userId);
        List<String> userIds = new ArrayList<>();
        if ("CompanyManager".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
        } else if ("bytedanceDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
        } else if ("kuaishouDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
        } else if (!"admin".equals(roleCode)) {
            userIds.add(userId);
        }
        kuaishouHostingTemplate.setUserId(null);
        String name = kuaishouHostingTemplate.getName();
        kuaishouHostingTemplate.setName(null);

        QueryWrapper<KuaishouHostingTemplate> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouHostingTemplate, req.getParameterMap());
        if (!Check.isNull(name)) {
            queryWrapper.like("name", name);
        }
        if (!Check.isNull(userIds)) {
            queryWrapper.in("user_id", userIds);
        }
        queryWrapper.orderByDesc("create_time");
        Page<KuaishouHostingTemplate> page = new Page<KuaishouHostingTemplate>(pageNo, pageSize);
        IPage<KuaishouHostingTemplate> pageList = kuaishouHostingTemplateService.page(page, queryWrapper);
        List<KuaishouHostingTemplate> records = pageList.getRecords();
        if (!Check.isNull(records)) {
            for (KuaishouHostingTemplate template : records) {
                UserAllocation userAllocation = userAllocationService.getByAccountId(template.getAccountId());
                if (Check.isNull(userAllocation)) {
                    continue;
                }
                template.setAccountName(userAllocation.getAuthName());
                template.setUserName(userAllocation.getUserName());
                Project project = projectService.getById(userAllocation.getProjectId());
                if (!Check.isNull(project)) {
                    template.setProjectName(project.getProjectName());
                }
            }
        }

        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaishouHostingTemplate
     * @return
     */
    @ApiOperation(value = "智能托管模板-添加", notes = "智能托管模板-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouHostingTemplate> add(@RequestBody KuaishouHostingTemplate kuaishouHostingTemplate) {
        Result<KuaishouHostingTemplate> result = new Result<>();
        try {
            kuaishouHostingTemplateService.save(kuaishouHostingTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouHostingTemplate
     * @return
     */
    @ApiOperation(value = "智能托管模板-编辑", notes = "智能托管模板-编辑")
    @PostMapping(value = "/edit")
    public Result<KuaishouHostingTemplate> edit(@RequestBody KuaishouHostingTemplate kuaishouHostingTemplate) {
        Result<KuaishouHostingTemplate> result = new Result<KuaishouHostingTemplate>();
        KuaishouHostingTemplate kuaishouHostingTemplateEntity = kuaishouHostingTemplateService.getById(kuaishouHostingTemplate.getId());
        if (kuaishouHostingTemplateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouHostingTemplateService.updateById(kuaishouHostingTemplate);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "智能托管模板-通过id删除", notes = "智能托管模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            kuaishouHostingTemplateService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "智能托管模板-批量删除", notes = "智能托管模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouHostingTemplate> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaishouHostingTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouHostingTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "智能托管模板-通过id查询", notes = "智能托管模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouHostingTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouHostingTemplate> result = new Result<>();
        KuaishouHostingTemplate kuaishouHostingTemplate = kuaishouHostingTemplateService.getById(id);
        if (kuaishouHostingTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouHostingTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouHostingTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouHostingTemplate kuaishouHostingTemplate = JSON.parseObject(deString, KuaishouHostingTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouHostingTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouHostingTemplate> pageList = kuaishouHostingTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "智能托管模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouHostingTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("智能托管模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouHostingTemplate> listKuaishouHostingTemplates = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouHostingTemplate.class, params);
                kuaishouHostingTemplateService.saveBatch(listKuaishouHostingTemplates);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouHostingTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
