package org.jeecg.modules.system.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.system.entity.SysOperateRel;
import org.jeecg.modules.system.service.ISysOperateRelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * 助理和运营关系记录表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2022-09-16
 */
@Slf4j
@Api(tags = "助理和运营关系记录表")
@RestController
@RequestMapping("/sys/operateRel")
public class SysOperateRelController {
    @Autowired
    private ISysOperateRelService sysOperateRelService;

    /**
     * 分页列表查询
     */
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(SysOperateRel sysOperateRel,
                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                        HttpServletRequest req) {
        try {
            PageHelper.startPage(pageNo, pageSize);
            return Result.ok(new PageInfo(sysOperateRelService.queryPageList(sysOperateRel)));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("success");
    }

    /**
     * 添加
     *
     * @param sysOperateRel
     * @return
     */
    @ApiOperation(value = "助理和运营关系记录表-添加", notes = "助理和运营关系记录表-添加")
    @PostMapping(value = "/add")
    public Result<SysOperateRel> add(@RequestBody SysOperateRel sysOperateRel) {
        Result<SysOperateRel> result = new Result<>();
        try {
            sysOperateRelService.add(sysOperateRel);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/delete")
    public Result<?> delete(String assistantId, String statMonth) {
        try {
            Map<String, Object> deleteMap = new HashMap<>();
            deleteMap.put("assistant_id", assistantId);
            deleteMap.put("stat_month", statMonth);
            sysOperateRelService.removeByMap(deleteMap);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "助理和运营关系记录表-批量删除", notes = "助理和运营关系记录表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<SysOperateRel> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<SysOperateRel> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.sysOperateRelService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "助理和运营关系记录表-通过id查询", notes = "助理和运营关系记录表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<SysOperateRel> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<SysOperateRel> result = new Result<>();
        SysOperateRel sysOperateRel = sysOperateRelService.getById(id);
        if (sysOperateRel == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(sysOperateRel);
            result.setSuccess(true);
        }
        return result;
    }

}
